@@ -46,26 +46,25 @@ import (
 	actuatoriface "github.com/openshift/cloud-credential-operator/pkg/operator/credentialsrequest/actuator"
 	awsannotator "github.com/openshift/cloud-credential-operator/pkg/operator/secretannotator/aws"
 	"github.com/openshift/cloud-credential-operator/pkg/operator/utils"
+	awsutils "github.com/openshift/cloud-credential-operator/pkg/operator/utils/aws"
 )
 
 const (
-	rootAWSCredsSecretNamespace = "kube-system"
-	rootAWSCredsSecret          = "aws-creds"
-	roAWSCredsSecretNamespace   = "openshift-cloud-credential-operator"
-	roAWSCredsSecret            = "cloud-credential-operator-iam-ro-creds"
-	clusterConfigNamespace      = "kube-system"
-	openshiftClusterIDKey       = "openshiftClusterID"
-	clusterVersionObjectName    = "version"
+	roAWSCredsSecretNamespace = "openshift-cloud-credential-operator"
+	roAWSCredsSecret          = "cloud-credential-operator-iam-ro-creds"
+	openshiftClusterIDKey     = "openshiftClusterID"
+	clusterVersionObjectName  = "version"
 )
 
 var _ actuatoriface.Actuator = (*AWSActuator)(nil)
 
 // AWSActuator implements the CredentialsRequest Actuator interface to create credentials in AWS.
 type AWSActuator struct {
-	Client           client.Client
-	Codec            *minterv1.ProviderCodec
-	AWSClientBuilder func(accessKeyID, secretAccessKey []byte, region, infraName string) (ccaws.Client, error)
-	Scheme           *runtime.Scheme
+	Client              client.Client
+	Codec               *minterv1.ProviderCodec
+	AWSClientBuilder    func(accessKeyID, secretAccessKey []byte, c client.Client) (ccaws.Client, error)
+	Scheme              *runtime.Scheme
+	testUpcomingSecrets []types.NamespacedName
 }
 
 // NewAWSActuator creates a new AWSActuator.
@@ -79,7 +78,7 @@ func NewAWSActuator(client client.Client, scheme *runtime.Scheme) (*AWSActuator,
 	return &AWSActuator{
 		Codec:            codec,
 		Client:           client,
-		AWSClientBuilder: ccaws.NewClient,
+		AWSClientBuilder: awsutils.ClientBuilder,
 		Scheme:           scheme,
 	}, nil
 }
@@ -142,7 +141,7 @@ func (a *AWSActuator) Exists(ctx context.Context, cr *minterv1.CredentialsReques
 
 // needsUpdate will return whether the current credentials satisfy what's being requested
 // in the CredentialsRequest
-func (a *AWSActuator) needsUpdate(ctx context.Context, cr *minterv1.CredentialsRequest, region, infraName string) (bool, error) {
+func (a *AWSActuator) needsUpdate(ctx context.Context, cr *minterv1.CredentialsRequest) (bool, error) {
 	logger := a.getLogger(cr)
 	// If the secret simply doesn't exist, we definitely need an update
 	exists, err := a.Exists(ctx, cr)
@@ -155,7 +154,7 @@ func (a *AWSActuator) needsUpdate(ctx context.Context, cr *minterv1.CredentialsR
 
 	// Various checks for the kinds of reasons that would trigger a needed update
 	_, accessKey, secretKey := a.loadExistingSecret(cr)
-	awsClient, err := a.AWSClientBuilder([]byte(accessKey), []byte(secretKey), region, infraName)
+	awsClient, err := a.AWSClientBuilder([]byte(accessKey), []byte(secretKey), a.Client)
 	if err != nil {
 		return true, err
 	}
@@ -169,7 +168,7 @@ func (a *AWSActuator) needsUpdate(ctx context.Context, cr *minterv1.CredentialsR
 		return true, fmt.Errorf("unable to decode ProviderStatus: %v", err)
 	}
 
-	readAWSClient, err := a.buildReadAWSClient(cr, region, infraName)
+	readAWSClient, err := a.buildReadAWSClient(cr)
 	if err != nil {
 		log.WithError(err).Error("error creating read-only AWS client")
 		return true, fmt.Errorf("unable to check whether AWS user is properly tagged")
@@ -191,6 +190,10 @@ func (a *AWSActuator) needsUpdate(ctx context.Context, cr *minterv1.CredentialsR
 			return true, err
 		}
 
+		infraName, err := utils.LoadInfrastructureName(a.Client, logger)
+		if err != nil {
+			return true, err
+		}
 		if !userHasExpectedTags(logger, user.User, infraName, string(clusterUUID)) {
 			return true, nil
 		}
@@ -234,7 +237,7 @@ func (a *AWSActuator) needsUpdate(ctx context.Context, cr *minterv1.CredentialsR
 		if mode == operatorv1.CloudCredentialsModePassthrough {
 			logger.Debug("will not perform permissions simulation because operator in mode %q", mode)
 		} else {
-			region, err := utils.LoadInfrastructureRegion(a.Client, logger)
+			region, err := awsutils.LoadInfrastructureRegion(a.Client, logger)
 			if err != nil {
 				return true, err
 			}
@@ -273,18 +276,8 @@ func (a *AWSActuator) sync(ctx context.Context, cr *minterv1.CredentialsRequest)
 	logger := a.getLogger(cr)
 	logger.Debug("running sync")
 
-	infraName, err := utils.LoadInfrastructureName(a.Client, logger)
-	if err != nil {
-		return err
-	}
-
-	region, err := utils.LoadInfrastructureRegion(a.Client, logger)
-	if err != nil {
-		return err
-	}
-
 	// Should we update anything
-	needsUpdate, err := a.needsUpdate(ctx, cr, region, infraName)
+	needsUpdate, err := a.needsUpdate(ctx, cr)
 	if err != nil {
 		logger.WithError(err).Error("error determining whether a credentials update is needed")
 		return &actuatoriface.ActuatorError{
@@ -320,7 +313,7 @@ func (a *AWSActuator) sync(ctx context.Context, cr *minterv1.CredentialsRequest)
 		}
 	case constants.MintAnnotation:
 		logger.Debugf("provisioning with cred minting")
-		err := a.syncMint(ctx, cr, region, infraName, logger)
+		err := a.syncMint(ctx, cr, logger)
 		if err != nil {
 			msg := "error syncing creds in mint-mode"
 			logger.WithError(err).Error(msg)
@@ -360,7 +353,7 @@ func (a *AWSActuator) syncPassthrough(ctx context.Context, cr *minterv1.Credenti
 }
 
 // syncMint handles both create and update idempotently.
-func (a *AWSActuator) syncMint(ctx context.Context, cr *minterv1.CredentialsRequest, region, infraName string, logger log.FieldLogger) error {
+func (a *AWSActuator) syncMint(ctx context.Context, cr *minterv1.CredentialsRequest, logger log.FieldLogger) error {
 	var err error
 
 	awsSpec, err := DecodeProviderSpec(a.Codec, cr)
@@ -373,6 +366,11 @@ func (a *AWSActuator) syncMint(ctx context.Context, cr *minterv1.CredentialsRequ
 		return err
 	}
 
+	infraName, err := utils.LoadInfrastructureName(a.Client, logger)
+	if err != nil {
+		return err
+	}
+
 	// Generate a randomized User for the credentials:
 	// TODO: check if the generated name is free
 	if awsStatus.User == "" {
@@ -398,12 +396,12 @@ func (a *AWSActuator) syncMint(ctx context.Context, cr *minterv1.CredentialsRequ
 		}
 	}
 
-	rootAWSClient, err := a.buildRootAWSClient(cr, region, infraName)
+	rootAWSClient, err := a.buildRootAWSClient(cr)
 	if err != nil {
 		logger.WithError(err).Warn("error building root AWS client, will error if one must be used")
 	}
 
-	readAWSClient, err := a.buildReadAWSClient(cr, region, infraName)
+	readAWSClient, err := a.buildReadAWSClient(cr)
 	if err != nil {
 		logger.WithError(err).Error("error building read-only AWS client")
 		return err
@@ -418,7 +416,7 @@ func (a *AWSActuator) syncMint(ctx context.Context, cr *minterv1.CredentialsRequ
 			case iam.ErrCodeNoSuchEntityException:
 				logger.WithField("userName", awsStatus.User).Debug("user does not exist, creating")
 				if rootAWSClient == nil {
-					return fmt.Errorf("no root AWS client available, cred secret may not exist: %s/%s", rootAWSCredsSecretNamespace, rootAWSCredsSecret)
+					return fmt.Errorf("no root AWS client available, cred secret may not exist: %s/%s", constants.CloudCredSecretNamespace, constants.AWSCloudCredSecretName)
 				}
 
 				createOut, err := a.createUser(logger, rootAWSClient, awsStatus.User)
@@ -447,7 +445,7 @@ func (a *AWSActuator) syncMint(ctx context.Context, cr *minterv1.CredentialsRequ
 	// Check if the user has the expected tags:
 	if !userHasExpectedTags(logger, userOut, infraName, string(clusterUUID)) {
 		if rootAWSClient == nil {
-			return fmt.Errorf("no root AWS client available, cred secret may not exist: %s/%s", rootAWSCredsSecretNamespace, rootAWSCredsSecret)
+			return fmt.Errorf("no root AWS client available, cred secret may not exist: %s/%s", constants.CloudCredSecretNamespace, constants.AWSCloudCredSecretName)
 		}
 
 		err = a.tagUser(logger, rootAWSClient, awsStatus.User, infraName, string(clusterUUID))
@@ -465,7 +463,7 @@ func (a *AWSActuator) syncMint(ctx context.Context, cr *minterv1.CredentialsRequ
 	policyEqual, err := a.awsPolicyEqualsDesiredPolicy(desiredUserPolicy, awsSpec, awsStatus, userOut, readAWSClient, logger)
 	if !policyEqual {
 		if rootAWSClient == nil {
-			return fmt.Errorf("no root AWS client available, cred secret may not exist: %s/%s", rootAWSCredsSecretNamespace, rootAWSCredsSecret)
+			return fmt.Errorf("no root AWS client available, cred secret may not exist: %s/%s", constants.CloudCredSecretNamespace, constants.AWSCloudCredSecretName)
 		}
 		err = a.setUserPolicy(logger, rootAWSClient, awsStatus.User, awsStatus.Policy, desiredUserPolicy)
 		if err != nil {
@@ -506,7 +504,7 @@ func (a *AWSActuator) syncMint(ctx context.Context, cr *minterv1.CredentialsRequ
 		// we should cleanup all pre-existing access keys. This will allow deleting the
 		// secret in Kubernetes to revoke old credentials and create new.
 		if rootAWSClient == nil {
-			return fmt.Errorf("no root AWS client available, cred secret may not exist: %s/%s", rootAWSCredsSecretNamespace, rootAWSCredsSecret)
+			return fmt.Errorf("no root AWS client available, cred secret may not exist: %s/%s", constants.CloudCredSecretNamespace, constants.AWSCloudCredSecretName)
 		}
 		err := a.deleteAllAccessKeys(logger, rootAWSClient, awsStatus.User, allUserKeys)
 		if err != nil {
@@ -618,16 +616,8 @@ func (a *AWSActuator) Delete(ctx context.Context, cr *minterv1.CredentialsReques
 	logger = logger.WithField("userName", awsStatus.User)
 
 	logger.Info("deleting credential from AWS")
-	region, err := utils.LoadInfrastructureRegion(a.Client, logger)
-	if err != nil {
-		return err
-	}
-	infraName, err := utils.LoadInfrastructureName(a.Client, logger)
-	if err != nil {
-		return err
-	}
 
-	awsClient, err := a.buildRootAWSClient(cr, region, infraName)
+	awsClient, err := a.buildRootAWSClient(cr)
 	if err != nil {
 		return err
 	}
@@ -753,19 +743,19 @@ func (a *AWSActuator) tagUser(logger log.FieldLogger, awsClient minteraws.Client
 
 // buildRootAWSClient will return an AWS client using the "root" AWS creds which are expected to
 // live in kube-system/aws-creds.
-func (a *AWSActuator) buildRootAWSClient(cr *minterv1.CredentialsRequest, region, infraName string) (minteraws.Client, error) {
-	logger := a.getLogger(cr).WithField("secret", fmt.Sprintf("%s/%s", rootAWSCredsSecretNamespace, rootAWSCredsSecret))
+func (a *AWSActuator) buildRootAWSClient(cr *minterv1.CredentialsRequest) (minteraws.Client, error) {
+	logger := a.getLogger(cr).WithField("secret", fmt.Sprintf("%s/%s", constants.CloudCredSecretNamespace, constants.AWSCloudCredSecretName))
 
 	logger.Debug("loading AWS credentials from secret")
 	// TODO: Running in a 4.0 cluster we expect this secret to exist. When we run in a Hive
 	// cluster, we need to load different secrets for each cluster.
-	accessKeyID, secretAccessKey, err := utils.LoadCredsFromSecret(a.Client, rootAWSCredsSecretNamespace, rootAWSCredsSecret)
+	accessKeyID, secretAccessKey, err := utils.LoadCredsFromSecret(a.Client, constants.CloudCredSecretNamespace, constants.AWSCloudCredSecretName)
 	if err != nil {
 		return nil, err
 	}
 
 	logger.Debug("creating root AWS client")
-	return a.AWSClientBuilder(accessKeyID, secretAccessKey, region, infraName)
+	return a.AWSClientBuilder(accessKeyID, secretAccessKey, a.Client)
 }
 
 // buildReadAWSCreds will return an AWS client using the the scaled down read only AWS creds
@@ -775,7 +765,7 @@ func (a *AWSActuator) buildRootAWSClient(cr *minterv1.CredentialsRequest, region
 //
 // If these are not available but root creds are, we will use the root creds instead.
 // This allows us to create the read creds initially.
-func (a *AWSActuator) buildReadAWSClient(cr *minterv1.CredentialsRequest, region, infraName string) (minteraws.Client, error) {
+func (a *AWSActuator) buildReadAWSClient(cr *minterv1.CredentialsRequest) (minteraws.Client, error) {
 	logger := a.getLogger(cr).WithField("secret", fmt.Sprintf("%s/%s", roAWSCredsSecretNamespace, roAWSCredsSecret))
 	logger.Debug("loading AWS credentials from secret")
 
@@ -788,12 +778,12 @@ func (a *AWSActuator) buildReadAWSClient(cr *minterv1.CredentialsRequest, region
 	if err != nil {
 		if errors.IsNotFound(err) {
 			logger.Warn("read-only creds not found, using root creds client")
-			return a.buildRootAWSClient(cr, region, infraName)
+			return a.buildRootAWSClient(cr)
 		}
 	}
 
 	logger.Debug("creating read AWS client")
-	client, err := a.AWSClientBuilder(accessKeyID, secretAccessKey, region, infraName)
+	client, err := a.AWSClientBuilder(accessKeyID, secretAccessKey, a.Client)
 	if err != nil {
 		return nil, err
 	}
@@ -812,7 +802,7 @@ func (a *AWSActuator) buildReadAWSClient(cr *minterv1.CredentialsRequest, region
 			switch aerr.Code() {
 			case "InvalidClientTokenId":
 				logger.Warn("InvalidClientTokenId for read-only AWS account, likely a propagation delay, falling back to root AWS client")
-				return a.buildRootAWSClient(cr, region, infraName)
+				return a.buildRootAWSClient(cr)
 			}
 			// Any other error we just let following code sort out.
 		}
@@ -905,12 +895,13 @@ func (a *AWSActuator) getDesiredUserPolicy(entries []minterv1.StatementEntry, us
 		Statement: []StatementEntry{},
 	}
 	for _, se := range entries {
-		policyDoc.Statement = append(policyDoc.Statement, StatementEntry{
-			Effect:    se.Effect,
-			Action:    se.Action,
-			Resource:  se.Resource,
-			Condition: se.PolicyCondition,
-		})
+		policyDoc.Statement = append(policyDoc.Statement,
+			StatementEntry{
+				Effect:    se.Effect,
+				Action:    se.Action,
+				Resource:  se.Resource,
+				Condition: se.PolicyCondition,
+			})
 	}
 
 	// Always allow a statment that enables iam:GetUser on yourself (to allow access_key/awsClient to username lookups)
@@ -923,9 +914,14 @@ func (a *AWSActuator) getDesiredUserPolicy(entries []minterv1.StatementEntry, us
 	return string(b), nil
 }
 
+// GetCredentialsRootSecretLocation returns the namespace and name where the parent credentials secret is stored.
+func (a *AWSActuator) GetCredentialsRootSecretLocation() types.NamespacedName {
+	return types.NamespacedName{Namespace: constants.CloudCredSecretNamespace, Name: constants.AWSCloudCredSecretName}
+}
+
 func (a *AWSActuator) getCloudCredentialsSecret(ctx context.Context, logger log.FieldLogger) (*corev1.Secret, error) {
 	cloudCredSecret := &corev1.Secret{}
-	if err := a.Client.Get(ctx, types.NamespacedName{Name: rootAWSCredsSecret, Namespace: rootAWSCredsSecretNamespace}, cloudCredSecret); err != nil {
+	if err := a.Client.Get(ctx, a.GetCredentialsRootSecretLocation(), cloudCredSecret); err != nil {
 		msg := "unable to fetch root cloud cred secret"
 		logger.WithError(err).Error(msg)
 		return nil, &actuatoriface.ActuatorError{
@@ -935,7 +931,7 @@ func (a *AWSActuator) getCloudCredentialsSecret(ctx context.Context, logger log.
 	}
 
 	if !isSecretAnnotated(cloudCredSecret) {
-		logger.WithField("secret", fmt.Sprintf("%s/%s", rootAWSCredsSecretNamespace, rootAWSCredsSecret)).Error("cloud cred secret not yet annotated")
+		logger.WithField("secret", fmt.Sprintf("%s/%s", constants.CloudCredSecretNamespace, constants.AWSCloudCredSecretName)).Error("cloud cred secret not yet annotated")
 		return nil, &actuatoriface.ActuatorError{
 			ErrReason: minterv1.CredentialsProvisionFailure,
 			Message:   fmt.Sprintf("cannot proceed without cloud cred secret annotation"),
@@ -1160,9 +1156,11 @@ type PolicyDocument struct {
 // StatementEntry is a simple type used to serialize to AWS' PolicyDocument format. We cannot
 // re-use minterv1.StatementEntry due to different conventions for the serialization keys. (caps)
 type StatementEntry struct {
-	Effect    string
-	Action    []string
-	Resource  string
+	Effect   string
+	Action   []string
+	Resource string
+	// Must "omitempty" otherwise we send unacceptable JSON to the AWS API when no
+	// condition is defined.
 	Condition minterv1.IAMPolicyCondition `json:",omitempty"`
 }
 
@@ -1204,7 +1202,7 @@ func (a *AWSActuator) Upgradeable(mode operatorv1.CloudCredentialsMode) *configv
 		Status: configv1.ConditionTrue,
 		Type:   configv1.OperatorUpgradeable,
 	}
-	toRelease := "4.6"
+	toRelease := "4.7"
 	if mode == operatorv1.CloudCredentialsModeManual {
 		// Check for the existence of credentials we know are coming in 4.6. If any do not exist, then we consider
 		// ourselves upgradable=false and inform the user why.
@@ -1243,7 +1241,7 @@ func (a *AWSActuator) Upgradeable(mode operatorv1.CloudCredentialsMode) *configv
 		}
 	} else {
 		// If in mint or passthrough, make sure the root cred secret exists, if not it must be restored prior to upgrade.
-		rootCred := types.NamespacedName{Namespace: rootAWSCredsSecretNamespace, Name: rootAWSCredsSecret}
+		rootCred := a.GetCredentialsRootSecretLocation()
 		secret := &corev1.Secret{}
 
 		err := a.Client.Get(context.Background(), rootCred, secret)
@@ -1272,9 +1270,11 @@ func (a *AWSActuator) Upgradeable(mode operatorv1.CloudCredentialsMode) *configv
 }
 
 func (a *AWSActuator) GetUpcomingCredSecrets() []types.NamespacedName {
+	// For unit testing only, otherwise we want these to be a static list.
+	if a.testUpcomingSecrets != nil {
+		return a.testUpcomingSecrets
+	}
 	return []types.NamespacedName{
-		// New in 4.6:
-		{Namespace: "openshift-cloud-credential-operator", Name: "cloud-credential-operator-s3"},
-		{Namespace: "openshift-cluster-csi-drivers", Name: "ebs-cloud-credentials"},
+		// Add new 4.7 credentials here, but none are known yet.
 	}
 }
